package org.ow2.jasmine.rules;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.drools.runtime.rule.FactHandle;
import org.ow2.jasmine.rules.api.IDroolsWorkingMemory;
import org.ow2.jasmine.rules.api.IDroolsWorkingMemoryPool;

/**
 * Implementation of {@link IDroolsWorkingMemoryPool}: a set of managed
 * Drools working memories.
 *
 * Not implemented in this abstract class:
 *  - {@link IDroolsWorkingMemoryPool#getOrCreate(String)}
 *
 * @author Julien Nicoulaud
 * @see org.ow2.jasmine.rules.osgi.impl.OSGiDroolsWorkingMemoryPool
 *
 */
public abstract class AbstractDroolsWorkingMemoryPool implements
        IDroolsWorkingMemoryPool {

    // -------------------------------------------------------------------------
    // Variables
    // -------------------------------------------------------------------------

    /**
     * The default working memory name, where packages without
     * any target working memory specified are added
     */
    public static final String DEFAULT_WORKING_MEMORY_NAME = "default";

    /**
     * The managed set: an association of (name, workingMemory)
     */
    private Map<String, IDroolsWorkingMemory> workingMemories =
        new HashMap<String, IDroolsWorkingMemory>();


    // -------------------------------------------------------------------------
    // Constructors
    // -------------------------------------------------------------------------

    /**
     * Create a managed pool of independent working memories.
     */
    public AbstractDroolsWorkingMemoryPool(){
    }


    // -------------------------------------------------------------------------
    // Set management
    // -------------------------------------------------------------------------

    /**
     * Add a WorkingMemory to the set, identified by a name.
     *
     * @param workingMemoryName the unique name
     * @param workingMemory the WorkingMemory
     * @throws Exception if the pool already contains a working memory
     * registered with the same name, the name is empty or the working memory
     * object is null.
     */
    public void add(String workingMemoryName,
                    IDroolsWorkingMemory workingMemory) throws Exception {

        // Check the validity of the name
        checkWorkingMemoryName(workingMemoryName);

        // No null object
        if (workingMemory == null) {
            throw new Exception("The working memory object to manage is null.");
        }

        // OK, add it
        else {
            workingMemories.put(workingMemoryName, workingMemory);
        }

    }

    /**
     * Remove a working memory from the set
     * Make sure to stop it before.
     *
     * @param workingMemoryName the targeted WorkingMemory
     * @return the WorkingMemory
     * @throws Exception if there is no working memory referenced
     * at that name.
     */
    public IDroolsWorkingMemory remove(String workingMemoryName) throws Exception{
        if(!workingMemories.containsKey(workingMemoryName)){
            throw new Exception(
                    "There is no working memory registered" +
                    "in the pool with the name " + workingMemoryName + ".");
        }else{
            return workingMemories.remove(workingMemoryName);
        }
    }

    /**
     * Get a WorkingMemory in the set
     *
     * @param workingMemoryName the targeted WorkingMemory
     * @return the WorkingMemory or null if it does not exist
     */
    public IDroolsWorkingMemory get(String workingMemoryName){
        return workingMemories.get(workingMemoryName);
    }


    // -------------------------------------------------------------------------
    // Working memories batch control
    // -------------------------------------------------------------------------

    /**
     * Start all managed working memories
     *
     * @throws Exception if a WorkingMemory fails to start
     */
    public void startAll() throws Exception {

        // Start working memories one by one
        for(IDroolsWorkingMemory theWorkingMemory: workingMemories.values()){
            theWorkingMemory.start();
        }
    }

    /**
     * Stop all managed working memories
     */
    public void stopAll() {

        // Stop working memories one by one
        for(IDroolsWorkingMemory theWorkingMemory: workingMemories.values()){
            theWorkingMemory.stop();
        }
    }

    /**
     * Insert a fact in all managed working memories
     * @param fact the fact object to insert
     * @return a map of <working memory name, FactHandle>
     */
    public HashMap<String,FactHandle> insertAll(Object fact) {

        HashMap<String,FactHandle> result = new HashMap<String,FactHandle>();

        // Insert the fact in the working memories one by one
        for(String theWorkingMemoryName: workingMemories.keySet()){
            FactHandle insertResult =
                ((IDroolsWorkingMemory) workingMemories.get(theWorkingMemoryName)).insert(fact);
            result.put(theWorkingMemoryName, insertResult);
        }
        return result;
    }


    // -------------------------------------------------------------------------
    // Helpers
    // -------------------------------------------------------------------------

    /*
     * (non-javadoc)
     * See org.ow2.jasmine.rules.osgi.impl.OSGiDroolsWorkingMemoryPool for
     * the implementation of getOrCreate()
     */


    // -------------------------------------------------------------------------
    // Informations & statistics getters
    // -------------------------------------------------------------------------

    /**
     * Get a list of the managed working memories names.
     *
     * @return a list of the memories names as strings
     */
    public List<String> getWorkingMemoriesNames(){

        List<String> result = new ArrayList<String>();
        for(Object name: workingMemories.keySet()){
            result.add((String) name);
        }
        return result;
    }


    // -------------------------------------------------------------------------
    // Private util methods
    // -------------------------------------------------------------------------

    /**
     * Check if a name is valid (not empty, null or already taken)
     *
     * @throws Exception if the name is empty, null or already taken.
     */
    private void checkWorkingMemoryName(String workingMemoryName) throws Exception{
        if (workingMemoryName == null || workingMemoryName == "") {
            // No empty names
            throw new Exception(
                    "A working memory cannot be added to the set " +
                    "with an empty name.");

        } else if (workingMemories.containsKey(workingMemoryName)) {
            // No duplicate names
            throw new Exception("The pool already contains a working memory" +
                    "registered with the name " + workingMemoryName + ".");
        }
    }
}
