/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.manager.bundle;

import java.lang.reflect.Proxy;
import java.util.List;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.ow2.jasmine.rules.manager.persistence.RulePackageSLBRemote;
import org.ow2.jasmine.rules.manager.service.DroolsManagerFactory;
import org.ow2.jasmine.rules.manager.service.IDroolsManagerFactory;
import org.ow2.jasmine.rules.manager.util.jndilocator.ServiceLocator;

import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Start and stop the rules manager bundle
 * @author Julien Nicoulaud
 */
public class RuleEngineManagerBundleActivator implements BundleActivator {
	
	// =========================================================================
	// Variables
	// =========================================================================
	
    /**
     * Logger
     */
    private static Log logger = LogFactory.getLog(RuleEngineManagerBundleActivator.class);

    /**
     * The rule engine manager factory service
     */
    public IDroolsManagerFactory droolsManagerFactory;
    
    /**
     * Keep a trace of the registration service in order to unregister it before stopping.
     */
    private ServiceRegistration serviceRegistration;

    /**
     * The persistence remote object
     */
    private RulePackageSLBRemote persistenceRemoteObject = null;
    
    
	// =========================================================================
	// Start
	// =========================================================================
    
    /**
     * Start the bundle
     */
    public void start(final BundleContext context) throws Exception {
    	
        // Log a message
    	logger.info("Rule engine manager starting");
    	
    	// Initialize the rule engine manager factory service
    	droolsManagerFactory = new DroolsManagerFactory(context);
    	
        // Register it as a service
        serviceRegistration = context.registerService(
			            		IDroolsManagerFactory.class.getName(),
			                    droolsManagerFactory,
			                    null);
        
        // Connect to the rules database
        try {
            Object obj = ServiceLocator.getInstance().getRemoteObject("db-ejb/rule");
            // uncomment these lines if problem of classloader to try to figure out this issue
            //logger.debug("db-ejb/rule class: " + obj.getClass());
            //logger.debug("getInvocationHandler " + Proxy.getInvocationHandler(obj));
            //logger.debug("CL=" + obj.getClass().getClassLoader());
            //logger.debug("RulePackageSLBRemote CL=" + RulePackageSLBRemote.class.getClassLoader());
            //logger.debug("thread CL=" + Thread.currentThread().getContextClassLoader());
            persistenceRemoteObject = (RulePackageSLBRemote) obj;
        } catch (Exception e) {
            throw(new Exception("Could not connect to the rules database",e));
        }
        
        // Show the number of rules saved
        logger.info(persistenceRemoteObject.countRulePackages() + " rule packages stored in the database");
        
        // Find the rule engine instances for which a manager needs to be setup
        List<String> ruleEngineInstances = persistenceRemoteObject.getRuleEngineInstancesToManage();
        for (String instanceName : ruleEngineInstances) {
        	droolsManagerFactory.getManager(instanceName);
        }
        
        // Log a success message
        logger.info("Rule engine manager started");
    }
    
    
	// =========================================================================
	// Stop
	// =========================================================================
    
    /**
     * Stop the bundle
     */
    public void stop(final BundleContext context) {
    	
        // Log a message
    	logger.info("Rule engine manager stopping");
    	
        // Unregister the rule engine manager factory service
        if(serviceRegistration != null) {
            serviceRegistration.unregister();
        }
        
        // Do not kill the managers created, but no new managers can be created.
        
        // Log a success message
        logger.info("Rule engine manager stopped");
    
    }
}
