/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.rules.manager.ejb;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.ow2.jasmine.rules.manager.model.AbstractRulePackage;
import org.ow2.jasmine.rules.manager.model.IRulePackage;
import org.ow2.jasmine.rules.manager.model.textual.TextualRulePackage;
import org.ow2.jasmine.rules.manager.model.visual.IRule;
import org.ow2.jasmine.rules.manager.model.visual.Rule;
import org.ow2.jasmine.rules.manager.model.visual.VisualRulePackage;
import org.ow2.jasmine.rules.manager.persistence.RulePackageSLBRemote;

/**
 * The stateless bean for {@link IRulePackage} objects.
 *
 * @author HAN Tianyi
 * @author Julien Nicoulaud
 */
@Stateless(mappedName = "db-ejb/rule")
@Remote(RulePackageSLBRemote.class)
public class RulePackageSLB implements RulePackageSLBRemote {

    // =========================================================================
    // Variables
    // =========================================================================

    /**
     * Entity manager used by this session bean.
     */
    @PersistenceContext
    private EntityManager entityManager = null;


    // =========================================================================
    // Public methods
    // =========================================================================

    /**
     * Save a rule package
     *
     * @param r
     *             the rule package for saving, two type of rule package
     *             visual and textual implement the interface IRulePackage
     * @throws Exception
     */
    public final void saveRulePackage(final IRulePackage r) throws Exception {

        // If this is an update, the old version has to be deleted
        deleteRulePackage(r);

        // Prepare the RulePackageEB object
        RulePackageEB rpEntityBean = new RulePackageEB();
        rpEntityBean.setDisplayName(r.getDisplayName());
        rpEntityBean.setState(r.getState());
        rpEntityBean.setSystemName(r.getSystemName());
        rpEntityBean.setDescription(r.getDescription());
        rpEntityBean.setRuleEngineInstanceName(r.getRuleEngineInstanceName());

        // Switch on type of the rule package
        if (r instanceof TextualRulePackage) {

            // Persist the visual rule package
            TextualRulePackageEB trpEntityBean = saveTextualRulePackage((TextualRulePackage) r);

            // Link it to the rule package
            rpEntityBean.setTextualRulePackageEB(trpEntityBean);

        } else if (r instanceof VisualRulePackage) {

            // Persist the visual rule package
            VisualRulePackageEB vrpEntityBean = saveVisualRulePackage((VisualRulePackage) r);

            // Link it to the rule package
            rpEntityBean.setVisualRulePackageEB(vrpEntityBean);

        }

        // Persist the rule package
        entityManager.persist(rpEntityBean);
    }

    /**
     * Delete a rule package
     *
     * @param r
     *          the rule package for deleting, two type of rule package
     *          visual and textual implement the interface IRulePackage
     * @throws Exception
     */
    public final void deleteRulePackage(final IRulePackage r) throws Exception {

        // Build the query to retrieve the rule package from the database
        String stringQuery = "SELECT e " +
                             "FROM RulePackageEB e " +
                             "WHERE e.systemName LIKE \'";

        // If the systemName has changed, use the old one the spot the rule
        // package record on the database
        if (r.getPreviousSystemName() != null) {
            stringQuery += r.getPreviousSystemName();
        } else {
            stringQuery += r.getSystemName();
        }
        stringQuery += "\' AND " +
                       "e.ruleEngineInstanceName LIKE \'" + r.getRuleEngineInstanceName() + "\'";

        try {
            // Execute the query
            RulePackageEB rpEntityBean = (RulePackageEB) entityManager.createQuery(
                    stringQuery).getSingleResult();

            // Delete the rule package if it was found
            if (rpEntityBean != null) {
                
                // Delete the visual or textual rule package
                if (r instanceof TextualRulePackage) {
                    entityManager.remove(rpEntityBean.getTextualRulePackageEB());
                } else if (r instanceof VisualRulePackage) {
                    entityManager.remove(rpEntityBean.getVisualRulePackageEB());
                }

                // Delete the rule package
                entityManager.remove(rpEntityBean);
            }

        } catch(NoResultException nre) {
            // Not found: do nothing
        }
    }

    /**
     * Implementation of the inherited method: looks for the beans
     * corresponding, converts the search into an array of RulePackage objects
     * and return them back.
     *
     * @see RulePackage[] getRules(Integer, String, String, String, String,
     *      String, String, String, Integer);
     *
     * @param id
     *            the ID of rule
     * @param systemName
     *            The unique system name of the rule package
     * @param displayName
     *            The displayed name of the rule package
     * @param description
     *            The description of the rule package
     * @param state
     *            The state of the rule package
     * @param orderBy
     *            Order using column.
     * @param limit
     *            Maximal number of results to get.
     *
     * @return Rules corresponding to the given search, null if none found.
     *
     * @throws Exception
     */
    @SuppressWarnings("unchecked")
    public final IRulePackage[] getRulePackages(final Integer id,
                                                final String systemName,
                                                final String displayName,
                                                final String ruleEngineInstanceName,
                                                final String description,
                                                final String state,
                                                final String orderBy,
                                                final Integer limit) throws Exception {

        HashMap<String, Object> queryElements = new HashMap<String, Object>();
        queryElements.put("id", id);
        queryElements.put("systemName", systemName);
        queryElements.put("displayName", displayName);
        queryElements.put("ruleEngineInstanceName",ruleEngineInstanceName);
        queryElements.put("description", description);
        queryElements.put("state", state);

        StringBuffer stringQuery = new StringBuffer(
                "SELECT e FROM RulePackageEB e");

        for (String key : queryElements.keySet()) {
            if (queryElements.get(key) != null) {
                if (stringQuery.length() > "SELECT e FROM RulePackageEB e"
                        .length()) {
                    stringQuery.append(" AND ");
                } else {
                    stringQuery.append(" WHERE ");
                }
                stringQuery.append("e.");
                stringQuery.append(key);
                stringQuery.append(" LIKE :");
                stringQuery.append(key);
            }
        }
        if (orderBy != null) {
            stringQuery.append(" ORDER BY e.");
            stringQuery.append(orderBy);
            stringQuery.append(" ASC");
        }

        Query dbQuery = entityManager.createQuery(stringQuery.toString());
        if (limit != null) {
            dbQuery.setMaxResults(limit);
        }
        for (String key : queryElements.keySet()) {
            if (queryElements.get(key) != null) {
                dbQuery.setParameter(key, queryElements.get(key));
            }
        }
        List<RulePackageEB> queryResult = dbQuery.getResultList();

        if (queryResult.size() < 1) {
            return null;
        } else {
            int i = 0;
            IRulePackage[] result = new IRulePackage[queryResult.size()];
            for (RulePackageEB bean : queryResult) {
                if (bean.getTextualRulePackageEB() != null) {
                    TextualRulePackage trp = new TextualRulePackage();
                    trp.setDisplayName(bean.getDisplayName());
                    trp.setRuleEngineInstanceName(bean.getRuleEngineInstanceName());
                    trp.setPreviousSystemName(null);
                    trp.setState(bean.getState());
                    trp.setSystemName(bean.getSystemName());
                    trp.setDescription(bean.getDescription());
                    trp.setContent(bean.getTextualRulePackageEB().getContent());
                    result[i++] = trp;
                } else if (bean.getVisualRulePackageEB() != null) {
                    VisualRulePackage vrp = new VisualRulePackage();
                    vrp.setDisplayName(bean.getDisplayName());
                    vrp.setRuleEngineInstanceName(bean.getRuleEngineInstanceName());
                    vrp.setPreviousSystemName(null);
                    vrp.setSystemName(bean.getSystemName());
                    vrp.setDescription(bean.getDescription());
                    vrp.setState(bean.getState());
                    vrp.setImports(bean.getVisualRulePackageEB().getImports());
                    List<IRule> lIRule = new ArrayList<IRule>();
                    List<Rule> lrEntityBean = bean.getVisualRulePackageEB().getRules();
                    for (Rule rule : lrEntityBean) {
                        lIRule.add(rule);
                    }
                    vrp.setRules(lIRule);
                    result[i++] = vrp;
                }
            }
            return result;
        }
    }

    /**
     * Get the number of rules
     *
     * @return size
     *              the number of rules
     */
    public final int countRulePackages() {
        // The query to get all the rule package from the database
        String stringQuery = "SELECT COUNT(e) FROM RulePackageEB e";

        // Execute the query
        Query dbQuery = entityManager.createQuery(stringQuery);

        // return the size of result
        return Integer.parseInt(dbQuery.getSingleResult().toString());
    }

    /**
     * Get the number of rules stored for a specific rule engine instance
     *
     * @return size
     *              the number of rules
     */
    public final int countRulePackages(final String ruleEngineInstanceName) {

        // The query to count the rule packages for a rule engine instance
        String stringQuery = "SELECT COUNT(e) " +
                             "FROM RulePackageEB e " +
                             "WHERE e.ruleEngineInstanceName LIKE \'" + ruleEngineInstanceName + "\'";

        // Execute the query
        Query dbQuery = entityManager.createQuery(stringQuery);

        // return the size of result
        return Integer.parseInt(dbQuery.getSingleResult().toString());
    }

    /**
     * Get the names of the target rule engine instances for which a manager
     * should be created. This the rule engine instances that have rules marked
     * as activated in the database.
     * 
     * @return List<String>
     *             return the table as a result
     */
    @SuppressWarnings("unchecked")
    public final List<String> getRuleEngineInstancesToManage() {

        // The query to get the unique result from the database
        String stringQuery = "SELECT DISTINCT e.ruleEngineInstanceName " +
                             "FROM RulePackageEB AS e " +
                             "WHERE e.state LIKE \'" + AbstractRulePackage.ACTIVATED + "\'";

        // Execute the query
        Query dbQuery = entityManager.createQuery(stringQuery);

        // Get the list in the result
        List<String> queryResult = dbQuery.getResultList();

        List<String> res = new ArrayList<String>();
        for (String s : queryResult) {
            if (res != null && !res.equals("null")) {
                res.add(s);
            }
        }
        // Return the table
        return res;
    }

    /**
     * Check a rule package name availability
     */
    public Boolean checkNameAvailability(String systemName,
                                  String ruleEngineInstanceName) throws Exception {

        // The query to count the rule packages matching the criteria
        String stringQuery = "SELECT COUNT(e) " +
                             "FROM RulePackageEB e " +
                             "WHERE e.systemName LIKE \'" + systemName + "\' " +
                             "AND e.ruleEngineInstanceName LIKE \'" + ruleEngineInstanceName + "\'";

        // Execute the query
        Query dbQuery = entityManager.createQuery(stringQuery);

        // Return true if no corresponding package was found
        return (Integer.parseInt(dbQuery.getSingleResult().toString()) == 0);
    }


    // =========================================================================
    // Private methods
    // =========================================================================

    /**
     * Implementation of inherited method: creates the Rule entity bean
     * corresponding to given data and saves the textual rule on the database.
     *
     * @param r
     *            Rule to save.
     * @return vrpEB
     *            Visual Rule for save
     */
    private TextualRulePackageEB saveTextualRulePackage(
            final TextualRulePackage r) throws Exception {

        // Create a instance of the entity bean Textual Rule Package
        TextualRulePackageEB trpEntityBean = new TextualRulePackageEB();

        // Prepare the TextualRulePackageEB object
        trpEntityBean.setContent(r.getContent());

        // Persist the textual rule package
        entityManager.persist(trpEntityBean);

        // return the entity bean
        return trpEntityBean;
    }

    /**
     * Implementation of inherited method: creates the Rule entity bean
     * corresponding to given data and saves the visual rule on the database.
     *
     * @param r
     *            Rule to save.
     * @return vrpEB
     *            Visual Rule for save
     */
    private VisualRulePackageEB saveVisualRulePackage(final VisualRulePackage r)
            throws Exception {

        // Create a instance of the entity bean Visual Rule Package
        VisualRulePackageEB vrpEntityBean = new VisualRulePackageEB();
        vrpEntityBean.setImports((ArrayList<String>)r.getImports());

        // Create an arraylist of entity bean Rule
        ArrayList<Rule> lRuleEntityBean = new ArrayList<Rule>();

        // Parse the rules
        for (IRule ir : r.getRules()) {
            lRuleEntityBean.add((Rule)ir);
        }

        // Add the entity bean for store
        vrpEntityBean.setRules(lRuleEntityBean);

        // Persist the visual rule package
        entityManager.persist(vrpEntityBean);

        return vrpEntityBean;
    }

}
